@@ -2,7 +2,6 @@
 # data: list object containing response y and group vector 'group'
 # fam_fun: family function output from 'family_export' function
 #' @importFrom lme4 lmer glmer VarCorr
-#' @importFrom coxme coxme VarCorr
 var_init = function(data, fam_fun){
   y = data$y
   grp = data$group
@@ -10,10 +9,6 @@ var_init = function(data, fam_fun){
     int_only = lmer(formula = y ~ 1 + (1 | grp))
   }else if(fam_fun$family %in% c("binomial","poisson")){
     int_only = glmer(formula = y ~ 1 + (1 | grp), family = fam_fun)
-  }else if(fam_fun$family == "coxph"){
-    y_times = data$y_times
-    y_status = data$y_status
-    int_only = coxme(formula = Surv(y_times, y_status) ~ (1 | grp))
   }
   
   var_start = VarCorr(int_only)$grp[[1]] * 2.0
@@ -32,3 +27,24 @@ var_init = function(data, fam_fun){
   return(var_start)
 }
 
+#' @importFrom lme4 lmer glmer VarCorr
+#' @importFrom stats poisson
+var_init_survival = function(data){
+  y = data$y
+  grp = data$group
+  cut_points = data$cut_points
+  cut_num = length(cut_points)
+  # Note: X1 inlcudes the reference coding for the time point indicators
+  X1 = data$X[,c(1:length(cut_points)), drop=FALSE]
+  # Since X1 includes an intercept term, remove automatic intercept from coding
+  int_only = glmer(formula = y ~ -1 + X1 + (1 | grp), family = poisson())
+  
+  var_start = VarCorr(int_only)$grp[[1]]
+  
+  # Protect against case when variance estimate above is 0 or very small
+  var_start = max(0.5, var_start) 
+  message(sprintf("recommended starting variance: %f", var_start))
+  
+  return(var_start)
+}
+
